package klax;

import java.awt.Color;
import java.awt.Graphics;

/**
 * @author Christian Grfe
 * Beinhaltet den Auffangkorb
 * In ihm werden die Steine aufgefangen und "verarbeitet"
 * 
 */
public class JavaKlax_Korb extends JavaKlax_GameObjekt
{
    /** Punkte die es fr einen Stein gibt */
    private static final int steinpunkte=25;
    
    /** Das Array des Korbes, hier liegen alle Steine.<br>
     * Gre 5x5 [spalte]x[zeile] <br>
     * 0,0 -> links oben
     */
    private JavaKlax_Stein[][] korb;
    
    /**
     * Der Konstruktor erzeugt das Array (5x5)<br>
     * hat das gleiche x wie der Trichter
     * @param y -> y Position auf dem Spielfeld
     */
    public JavaKlax_Korb(int y)
    {
    	super(leftX,y);
        korb=new JavaKlax_Stein[5][5];
    }
    
    /**
     * bergibt dem Korb einen Stein
     * @param stein -> der Stein der bergeben wird
     * @return Punkte die entstanden sind oder <br>
     * 	-1 wenn keine Steine weiter aufgenommen werden knnen
     */
    public int getStein(JavaKlax_Stein stein)
    {
    	int pos = stein.getSpalte();
    	int punkte;
    	
        // wenn hier kein Platz mehr fr den Stein ist
        if(korb[pos][0]!=null) return -1;
        
        // Stein in diese Spalte aufnehmen
        korb[pos][0]=stein;
        
        // den Stein nach unten nachrcken
        for(int i=korb[pos].length-1;i>0;i--)
            if(korb[pos][i]==null)
            {
                korb[pos][i]=stein;
                korb[pos][0]=null;
                break;
            }
       
        // auf punkte testen
        punkte=punkteTest();
            
        // die y-koordinaten der steine auf den neusten stand bringen
        updateSteinPos();
        
        return punkte; 
    }
    
    /**
     * Testet den Korb ob Reihen entstanden sind und sich so Punkte ergeben haben
     * @return Punkte, die entstanden sind
     */
    private int punkteTest()
    {
        int punkte = 0,i,j;
        // TODO Steine Diagonal berprfen
        
        // Steine Horizontal berprfen
        if(punkte==0)
        {
            for(j=0;j<5;j++)		// zeile
                for(i=0;i<3;i++)	// spalte
                   if(korb[i][j]!=null && korb[i+1][j]!=null && korb[i+2][j]!=null)
                        if(korb[i][j].getColor()==korb[i+1][j].getColor() && korb[i][j].getColor()==korb[i+2][j].getColor())
                        {
                            punkte=3*steinpunkte;
                            if(i<2)
                                if(korb[i+3][j]!=null)
                                    if(korb[i][j].getColor()==korb[i+3][j].getColor())
                                    {
                                        punkte*=2;
                                        if(i==0)
                                            if(korb[i+4][j]!=null)
                                                if(korb[i][j].getColor()==korb[i+4][j].getColor())
                                                {
                                                    punkte*=4;
                                                    korb[i+4][j]=null;
                                                }
                                        
                                        korb[i+3][j]=null;
                                    }
                            korb[i][j]=korb[i+1][j]=korb[i+2][j]=null;   
                        }
        }
        
        // Steine Waagerecht berprfen
        if(punkte==0)
        {
            for(i=0;i<5;i++)
                for(j=0;j<3;j++)
                    if(korb[i][j]!=null && korb[i][j+1]!=null && korb[i][j+2]!=null)
                        if(korb[i][j].getColor()==korb[i][j+1].getColor() && korb[i][j].getColor()==korb[i][j+2].getColor())
                        {
                            punkte=3*steinpunkte;
                            if(j<2)
                                if(korb[i][j+3]!=null)
                                    if(korb[i][j].getColor()==korb[i][j+3].getColor())
                                    {
                                        punkte*=2;
                                        korb[i][j+3]=null;
                                    }
                            korb[i][j]=korb[i][j+1]=korb[i][j+2]=null;   
                        }
        }
        
        // das Feld Nachrcken
        if(nachruecken()) punkte+=4*punkteTest();
        
        return punkte;
    }
    
    /**
     * Sorgt dafr, das alle Steine im Korb nach unten nachrcken.<br>
     * Es gibt keine Lcken mehr im Feld.
     * @return true  -> es wurde etwas gendert<br>
     * false -> das Feld ist genau so, wie vor der Funktion
     */
    private boolean nachruecken()
    {
        boolean aenderung=false;
        int i,j,k;
        // alle Steine im Korb nachrcken
        for (i=0;i<5;i++)	// alle spalten
            for(j=4;j>0;j--)// alle zeilen auser der obersten
                if(korb[i][j]==null) // nix da?
                    for(k=j-1;k>=0;k--) // is drber was das
                        if(korb[i][k]!=null) 
                        {
                            korb[i][j]=korb[i][k];
                            korb[i][k]=null;
                            aenderung=true;
                            break;
                        }
        
        return aenderung;
    }
    
    /** zeichnet den Korb und seine Steine */
    public void draw(Graphics g)
    {
    	// Korb Zeichnen
        g.setColor(Color.BLUE);
	    // Zwischenlinien
	    int x_pos=0, y_bottom=y+5*JavaKlax_Stein.hoehe;
	    for(int i=0;i<=korb.length;i++)
	    {
	        x_pos=leftX+i*(JavaKlax_Stein.breite+6);
	        g.drawLine(x_pos,y,x_pos,y_bottom);
	    }
	    g.drawLine(x,y_bottom,x_pos,y_bottom);
    	
    	// Steine des Korbes Zeichnen
    	int i,j;
    	for(i=0;i<korb.length;i++)
    		for(j=0;j<korb[i].length;j++)
    			if(korb[i][j]!=null)korb[i][j].draw(g);
    }
    
    /** Setz die Y-Positionen der Steine des Korbes auf den neuesten Stand */
    private void updateSteinPos()
    {
        int i,j;
    	for(i=0;i<korb.length;i++)
    		for(j=0;j<korb[i].length;j++)
    			if(korb[i][j]!=null)korb[i][j].setY(y+j*JavaKlax_Stein.hoehe);
    }
}
