package klax;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Timer;

/**
 * @author Christian Grfe
 * 
 * Diese Klasse sorgt dafr, dass das Spiel als Application ausgefhrt wird
 * (Game-Machine)
 */

public class JavaKlax_Machine extends Frame implements KeyListener,MouseListener
{
    /** Die Spiel-Engine */
    private JavaKlax_Engine game;
    /** Der Timer fr das Spiel -> die Clock des Spiels */
    private Timer tick;
    /** luft das Spiel oder ist das Men Aktiv */
    private boolean game_activ;
    
    // Elemente fr das Men
    /** Das Hauptpanel, hier liegt das ganze Men drin */
    private Panel pMenu;
    /** Der LayoutManager des Mens */
    CardLayout pMenuLayout;
    /** Das Panel fr die Men-unterteilung */
	Panel pMenuStruk;
	
	// der BackBuffer
	/** Das Image fr den BackBuffer */
	Image backbuffer_i;
	/** der Grafikkontext des BackBuffers */
    Graphics backbuffer_gc;
    
	/** 
	 * Konstruktor der Game-Maschine<br>
	 * Erzeugt das Fenster und das Men
	 * @param name -> Name des Fensters
	 */
	public JavaKlax_Machine(String name)
	{
		super(name);
		
		int i;
		
		// Fenster init
		this.setBackground(Color.BLACK);
		this.setForeground(Color.RED);
		this.setFont(new Font("SansSerif",Font.BOLD,16));
		this.setSize(380,500);
		this.setResizable(false);
		this.setLayout(new BorderLayout());
		
		// auf dem Bildschirm zentrieren
		Dimension d=getToolkit().getScreenSize();	 // Bildschrimgre erhalten
		this.setLocation(d.width/2-this.getSize().width/2,
				         d.height/2-this.getSize().height/2);
		
		// exit Hndler
		this.addWindowListener(new WindowAdapter(){public void windowClosing(WindowEvent e){System.exit(0);}});
		this.addKeyListener(this);
		
		// Panel fr das ganze Men erzeugen
		pMenu = new Panel(new BorderLayout());
		this.add(BorderLayout.CENTER,pMenu);
		
		// Panel fr die Menverteilung + Game Titel
		pMenuLayout = new CardLayout();
		pMenuStruk = new Panel(pMenuLayout);
		pMenu.add(BorderLayout.NORTH,new Label("JavaKlax",Label.CENTER));
		pMenu.add(BorderLayout.CENTER,pMenuStruk);
		
		// Panel fr das HauptMen und die Untermens
		Panel pHMenu=new Panel(new GridLayout(10,1));
		Panel[] pUmenu=new Panel[4];
		pUmenu[0] = new Panel();	// Einstellungen 
		pUmenu[1] = new Panel();	// High Score
		pUmenu[2] = new Panel();	// Spielregeln
		pUmenu[3] = new Panel();	// Infos
		
		TextArea[] text=new TextArea[pUmenu.length];
		for(i=0;i<pUmenu.length;i++)
		{
		    text[i]=new TextArea("Hier ist leider noch nichts! :(",10,50,TextArea.SCROLLBARS_NONE);
		    text[i].setBackground(Color.BLACK);
		    text[i].setEditable(false);
		    
		    Label lblBack=new Label("zurck",Label.CENTER);
			lblBack.addMouseListener(this);
			
		    pUmenu[i].setLayout(new BorderLayout());
		    pUmenu[i].add(BorderLayout.SOUTH, lblBack);
		    pUmenu[i].add(BorderLayout.CENTER, text[i]);
		}
		
		// Haupt und Untermens in die Menstruktur einbinden
		pMenuStruk.add("HMenu",pHMenu);
		pMenuStruk.add("Einstellungen",pUmenu[0]);
		pMenuStruk.add("High Score",pUmenu[1]);
		pMenuStruk.add("Spielregeln",pUmenu[2]);
		pMenuStruk.add("Infos",pUmenu[3]);
		pMenuLayout.show(pMenuStruk,"HMenu");
		
		// Menlabels erzeugen
		Label[] lblHMenu = new Label[6];
		lblHMenu[0] = new Label("Spiel starten",Label.CENTER);
		lblHMenu[1] = new Label("Einstellungen",Label.CENTER);
		lblHMenu[2] = new Label("High Score",Label.CENTER);
		lblHMenu[3] = new Label("Spielregeln",Label.CENTER);
		lblHMenu[4] = new Label("Infos",Label.CENTER);
		lblHMenu[5] = new Label("Ende",Label.CENTER);

		
		// Um das ganze nach unten zu verschieben
		pHMenu.add(new Panel());
		pHMenu.add(new Panel());
		for(i=0;i<lblHMenu.length;i++)
		{
		    lblHMenu[i].addMouseListener(this);
			pHMenu.add(BorderLayout.CENTER,lblHMenu[i]);
		}
	}
	
	/**
	 * Die MainFunktion von allem.<br>
	 * Einstiegstpunkt von Auserhalb.
	 * @param args nutzlos, bewirkt nichts
	 */
	public static void main(String[] args)
	{
		// Fenster erzeugen
		JavaKlax_Machine f=new JavaKlax_Machine("JavaKlax - Version 0.9");
		f.setVisible(true);
		// damit der BackBuffer gleich nach dem Start erzeugt wird
		f.repaint();
	}
	
	public void update(Graphics g)
	{
	    // wenn noch kein BackBuffer ezeugt wurde, einen erzeugen
	    // das steht leider hier, weil der BB in main oder dem Konstr. nicht erzeugt werden kann
	    if(backbuffer_i==null)
	    {
	        // BackBuffer erzeugen
			backbuffer_i = createImage(getSize().width,getSize().height);
		    backbuffer_gc = backbuffer_i.getGraphics();
	    }
	    
	    if(game_activ)
	    {
	        // BackBuffer schwarz zeichnen
	        backbuffer_gc.setColor(Color.BLACK);
	        backbuffer_gc.clearRect(0,0,getSize().width,getSize().height);
	        // Spiel auf den BB zeichnen
	        game.draw(backbuffer_gc);
	    }
	    
	    paint(g);
	}
	
	
	/**
	 * Die berladene paint-Methode des Fensters<br>
	 * wird bentigt um die Spiel-Engine zu zeichnen
	 */
	public void paint(Graphics g)
	{	
	    // wenn das Spiel activ-> BackBuffer auf den Screen
	    if(game_activ) g.drawImage(backbuffer_i,0,0,this);
	}
	
	/** Abfangen des Tastendrucks zur Steuerung der Spielfigur */
    public void keyPressed(KeyEvent k)
    {
        // nur wenn das spiel activ ist
        if (game_activ)
        {
	        // taste weiterleiten
	        switch(k.getKeyCode())
	        {
	        	case KeyEvent.VK_LEFT: game.press(JavaKlax_Engine.K_LEFT); break;
	        	case KeyEvent.VK_RIGHT: game.press(JavaKlax_Engine.K_RIGHT); break;
	        	case KeyEvent.VK_UP: game.press(JavaKlax_Engine.K_DROP_LIFO); break;
	        	case KeyEvent.VK_DOWN: game.press(JavaKlax_Engine.K_DROP_NORM); break;
	        	case KeyEvent.VK_P: game.press(JavaKlax_Engine.K_PAUSE); break;
	        	case KeyEvent.VK_ESCAPE:
	        	    tick.cancel();	// Timer lschen
	        		game=null;	// pointer zu spiel lsen
	        		System.gc(); // Spiel aus Speicher lschen
	        		game_activ=false;	// Men wieder einschalten
	        		pMenu.setVisible(true);
	        }
	        // neu zeichnen
	        this.repaint();
        }
        else if(k.getKeyCode()==KeyEvent.VK_ESCAPE) System.exit(0);
    }
    
    /** Abfangen des Maus-Klicks zur Navigation im Men */
    public void mouseClicked(MouseEvent e)
    {
        // das Objekt welches das Event auslst bekommen
        Label test= (Label)e.getSource();
        
        // labels des hauptmen testn
        if(test.getText()=="Ende") System.exit(0);
        else if(test.getText()=="Spiel starten")
        {
            // timer erzeugen, Spielactiv schalten
            pMenu.setVisible(false);
            game=new JavaKlax_Engine((byte)5,this);
    		tick= new Timer();
    		tick.schedule(game,100,400);
    		game_activ=true;
        }
        else if(test.getText()=="zurck") pMenuLayout.show(pMenuStruk,"HMenu");
        else pMenuLayout.show(pMenuStruk,test.getText());
    }
    
    
    /** wird nicht bentigt */
    public void keyReleased(KeyEvent k){}
    /** wird nicht bentigt */
    public void keyTyped(KeyEvent k){}
    /** wird nicht bentigt */
    public void mousePressed(MouseEvent e) {}
    /** wird nicht bentigt */
    public void mouseReleased(MouseEvent e) {}
    /** wird nicht bentigt */
    public void mouseEntered(MouseEvent e) {}
    /** wird nicht bentigt */
    public void mouseExited(MouseEvent e) {}
}
