package klax;

import java.awt.Color;
import java.awt.Graphics;

/**
 * @author Christian Grfe
 * 
 * Der Spieler mit seinen Eigenschaften und Funktionen
 */

public class JavaKlax_Player extends JavaKlax_GameObjekt
{
    /** Anzahl der Restlichen Leben */
    private byte leben;
    /** Die Position des Spielers (0 bis 4) */
    private byte pos;
    /** der Aktuelle Punktestand */
    private int punkte;
    /** Der Steinehalter fr 3 Steine -> 0=erster, 1=zweiter, 2=dritter Stein*/
    private JavaKlax_Stein[] halter;
    /** Anzahl der Steine im Steinehalter */
    private byte anz_steine;
    /** Breite des Spielers -> abhngig von der Steinbreite */
    private static final int breite = JavaKlax_Stein.breite+9;
    /** Hhe des Spielers */
    private static final int hoehe = 8;
    
    /**
     * Der Konstruktor des Spielers
     * @param leben Anzahl der Leben am Anfang des Spiels
     * @param pos Position des Spielers beim Start (0 bis 4 mglich, sonst 0)
     * @param y -> y Position des Spielers
     */ 
    public JavaKlax_Player(byte leben, byte pos, int y)
    {
        this.y=y;
        this.leben=leben;
        if(pos<5 && pos>=0) this.pos=pos;
        // X-Position aus Spalte berechnen
        x=leftX+pos*(JavaKlax_Stein.breite+6)-1;
        halter = new JavaKlax_Stein[3];
    }
    
    /**
     * Bewegt den Spieler nach Links oder Rechts
     * @param left true -> links ... false -> rechts
     */
    public void move(boolean left)
    {
        if(left)
        {
            if(pos>0) pos--;
        }
        else
        {
            if(pos<4) pos++;
        }
        
        // Spalte bei den Steinen auf dem Halter neu setzen
        for(int i=0;i<3;i++)
        	if(halter[i]!=null) halter[i].setSpalte(pos);
        	
        // x neu berechnen
        x=leftX+pos*(JavaKlax_Stein.breite+6)-1;
    }
    
    /**
     * Gibt die aktuelle Spaltenposition des Spielers zurck
     * @return Spalte in der sich der Spieler befindet
     */
    public byte getSpalte()
    {
    	return pos;
    }
    
    /**
     * Gibt den Aktuellen Punktestand zurck
     * @return Punkte des Spielers
     */
    public int getPunkte()
    {
        return punkte;
    }
    
    /**
     * Fgt dem Spieler Punkte hinzu
     * @param p Punkte die dazu kommen sollen
     */
    public void addPunkte(int p)
    {
        punkte+=p;
    }
    
    /**
     * Gibt die Anzahl an noch mglichen Leben zurck
     * @return leben des Spielers
     */
    public byte getLeben()
    {
    	return leben;
    }
    
    /** Der Spieler erhlt ein Leben dazu */
    public void addLeben()
    {
        leben++;
    }
    
    /**
     * Nimmt dem Spieler ein Leben
     * @return false -> Spieler ist tot <br>true  -> es wurde ein Leben abgezogen
     */
    public boolean killLeben()
    {
        if(leben==0) return false;
        leben--;
        return true;
    }
    
    /**
     * Der Spieler nimmt einen Stein auf seinen Halter auf
     * @param stein -> der Stein der aufgenommen wird 
     * @return false-> es kann kein Stein aufgenommen werden,<br>
     * 				   oder der Stein ist in der falschen Spalte<br>
     * 		   true -> Stein aufgenommen
     */
    public boolean getStein(JavaKlax_Stein stein)
    {
        // kann er berhaupt noch ein Stein aufnehmen
        if(anz_steine>2) return false;
        
        // ist der stein in der richtigen spalte?
        if(stein.getSpalte()!=pos) return false;
        
        // den Stein dem Spieler bergeben
        halter[anz_steine]=stein;
        anz_steine++;
        
        // die y-position der Steine neu setzen
        for(int i=0;i<anz_steine;i++)
            halter[i].setY(y+JavaKlax_Stein.hoehe*(anz_steine-1-i));
        
        // dem Spieler Punkte fr den Stein gutschreiben
        punkte+=5;
        
        return true;
    }
    
    /**
     * Nimmt einen Stein vom Halter des Spielers
     * @param lifo -> true -> oberster Stein wird abgenommen<br>-> false-> unterster Stein wird genommen
     * @return stein oder null wenn kein stein mehr da
     */
    public JavaKlax_Stein dropStein(boolean lifo)
    {
        if(anz_steine==0) return null;
        
        anz_steine--;
        JavaKlax_Stein tmp;
        
        if(lifo)
        {
            tmp=halter[anz_steine];
            halter[anz_steine]=null;
        }
        else
        {
            tmp=halter[0];
            halter[0]=halter[1];
            halter[1]=halter[2];
            halter[2]=null;
        }
        
        // Position der Steine neu bestimmen
        for(int i=0;i<anz_steine;i++)
            halter[i].setY(y+JavaKlax_Stein.hoehe*(anz_steine-1-i));
        
        // Stein zurckgeben
        return tmp;
    }
    
    /**
     * Zeichnet den Spieler
     * @param g Der Grafikkontext auf den gezeichnet wird
     */
    public void draw(Graphics g)
    {
        // Spieler zeichnen
        g.setColor(Color.WHITE);
    	g.fillRoundRect(x,y+anz_steine*JavaKlax_Stein.hoehe,breite,hoehe,breite>>2,hoehe>>1);
    	
    	// seine Steine zeichnen
    	for(int i=0;i<anz_steine;i++)
    	    halter[i].draw(g);
    }
}
